import React from 'react'
import { mount } from 'enzyme'
import { Provider } from 'react-redux'

import configureStore from 'redux-mock-store'
import {
  mockInitialChartState,
  mockInitialEcolyoState,
} from '../../../tests/__mocks__/store'
import DataloadConsumptionVisualizer from './DataloadConsumptionVisualizer'
import { FluidType } from 'enum/fluid.enum'
import { baseDataLoad } from '../../../tests/__mocks__/datachartData.mock'
import { Dataload } from 'models'
import { BrowserRouter as Router } from 'react-router-dom'
import UsageEventService from 'services/usageEvent.service'

jest.mock('cozy-ui/transpiled/react/I18n', () => {
  return {
    useI18n: jest.fn(() => {
      return {
        t: (str: string) => str,
      }
    }),
  }
})

const mockStore = configureStore([])
const mockChartStateLoaded = { ...mockInitialChartState, loading: false }
const emptyDataLoad = { ...baseDataLoad, value: -1 }
const dataLoadWithValueDetailEmpty: Dataload = {
  ...baseDataLoad,
  valueDetail: [],
}
const dataLoadWithValueDetail: Dataload = {
  ...baseDataLoad,
  valueDetail: [0.612216, 0.675242, 0.6779610000000003],
}

describe('Dataload consumption visualizer component', () => {
  it('should render correctly', () => {
    const store = mockStore({
      ecolyo: mockInitialEcolyoState,
    })
    const wrapper = mount(
      <Provider store={store}>
        <DataloadConsumptionVisualizer
          fluidType={FluidType.MULTIFLUID}
          dataload={baseDataLoad}
          showCompare={false}
          compareDataload={baseDataLoad}
          setActive={jest.fn()}
        />
      </Provider>
    )
    expect(wrapper).toMatchSnapshot()
  })
  it('should render with single fluid', () => {
    const store = mockStore({
      ecolyo: {
        chart: mockChartStateLoaded,
      },
    })
    const wrapper = mount(
      <Provider store={store}>
        <DataloadConsumptionVisualizer
          fluidType={FluidType.ELECTRICITY}
          dataload={baseDataLoad}
          showCompare={false}
          compareDataload={baseDataLoad}
          setActive={jest.fn()}
        />
      </Provider>
    )
    expect(wrapper).toMatchSnapshot()
  })
  it('should render with no value to compare available', async () => {
    const store = mockStore({
      ecolyo: {
        chart: mockChartStateLoaded,
      },
    })
    const wrapper = mount(
      <Provider store={store}>
        <DataloadConsumptionVisualizer
          fluidType={FluidType.WATER}
          dataload={baseDataLoad}
          showCompare={true}
          compareDataload={emptyDataLoad}
          setActive={jest.fn()}
        />
      </Provider>
    )
    expect(wrapper.find('.dataloadvisualizer-novalue').exists()).toBeTruthy()
  })
  it('should render with water comparison data', async () => {
    const store = mockStore({
      ecolyo: {
        chart: mockChartStateLoaded,
      },
    })
    const wrapper = mount(
      <Provider store={store}>
        <DataloadConsumptionVisualizer
          fluidType={FluidType.WATER}
          dataload={baseDataLoad}
          showCompare={true}
          compareDataload={baseDataLoad}
          setActive={jest.fn()}
        />
      </Provider>
    )
    expect(wrapper.find('.water-compare').exists()).toBeTruthy()
  })
  it('should render multifluid with no compare and display estimation modal', async () => {
    const store = mockStore({
      ecolyo: {
        chart: mockChartStateLoaded,
      },
    })

    const wrapper = mount(
      <Provider store={store}>
        <DataloadConsumptionVisualizer
          fluidType={FluidType.MULTIFLUID}
          dataload={dataLoadWithValueDetailEmpty}
          showCompare={false}
          compareDataload={emptyDataLoad}
          setActive={jest.fn()}
        />
      </Provider>
    )
    expect(wrapper.find('.estimated').first().simulate('click'))
  })
  it('should render multifluid with no compare and navigate to singleFluid page', async () => {
    const store = mockStore({
      ecolyo: {
        chart: mockChartStateLoaded,
      },
    })
    const mockLoadToEuro = jest.fn()
    jest.mock('services/converter.service', () => {
      return jest.fn(() => {
        return {
          LoadToEuro: mockLoadToEuro,
        }
      })
    })

    const wrapper = mount(
      <Provider store={store}>
        <Router>
          <DataloadConsumptionVisualizer
            fluidType={FluidType.MULTIFLUID}
            dataload={dataLoadWithValueDetail}
            showCompare={false}
            compareDataload={emptyDataLoad}
            setActive={jest.fn()}
          />
        </Router>
      </Provider>
    )
    jest.mock('services/usageEvent.service')
    const mockAddEvent = jest.fn()
    UsageEventService.addEvent = mockAddEvent

    // Render Navlinks to fluids
    wrapper.find('.dataloadvisualizer-euro-fluid').first().simulate('click')
    expect(mockAddEvent).toHaveBeenCalled()
  })
})
