import ConsumptionValidatorService from './consumptionValidator.service'
import { TimeStep } from 'enum/timeStep.enum'
import { DateTime } from 'luxon'
import { FluidType } from 'enum/fluid.enum'
import { TimePeriod } from 'models'

const mockTimePeriod: TimePeriod = {
  startDate: DateTime.fromISO('2020-10-01T00:00:00.000Z', { zone: 'utc' }),
  endDate: DateTime.fromISO('2020-10-03T23:59:59.999Z', { zone: 'utc' }),
}
const mockTimePeriodComparison: TimePeriod = {
  startDate: DateTime.fromISO('2020-09-01T00:00:00.000Z', { zone: 'utc' }),
  endDate: DateTime.fromISO('2020-09-03T23:59:59.999Z', { zone: 'utc' }),
}
let fluidTypes: FluidType[] = [0, 1, 2]
describe('ConsumptionFormatter service', () => {
  const consumptionValidatorService = new ConsumptionValidatorService()
  describe('formatGraphData method', () => {
    it('should return true for DAY', () => {
      const result = consumptionValidatorService.ValidateGetGraphData(
        mockTimePeriod,
        TimeStep.DAY,
        fluidTypes
      )
      expect(result).toBeTruthy()
    })
    it('should return true with comparison Date', () => {
      const result = consumptionValidatorService.ValidateGetGraphData(
        mockTimePeriod,
        TimeStep.DAY,
        fluidTypes,
        mockTimePeriodComparison
      )
      expect(result).toBeTruthy()
    })
    it('should return false with comparison Date', () => {
      const wrongTimePeriodComparison: TimePeriod = {
        startDate: DateTime.fromISO('2020-09-05T00:00:00.000Z', {
          zone: 'utc',
        }),
        endDate: DateTime.fromISO('2020-09-03T23:59:59.999Z', { zone: 'utc' }),
      }
      const result = consumptionValidatorService.ValidateGetGraphData(
        mockTimePeriod,
        TimeStep.DAY,
        fluidTypes,
        wrongTimePeriodComparison
      )
      expect(result).toBeFalsy()
    })
    it('should return false with comparison Date', () => {
      const wrongTimePeriodComparison: TimePeriod = {
        startDate: DateTime.fromISO('2020-09-05T00:00:00.000Z', {
          zone: 'utc',
        }),
        endDate: DateTime.fromISO('2020-09-07T23:59:59.999Z', { zone: 'utc' }),
      }
      const result = consumptionValidatorService.ValidateGetGraphData(
        mockTimePeriod,
        TimeStep.HALF_AN_HOUR,
        [0],
        wrongTimePeriodComparison
      )
      expect(result).toBeFalsy()
    })
    it('should return false because of fluid', () => {
      const wrongFluidType: FluidType[] = []
      const result = consumptionValidatorService.ValidateGetGraphData(
        mockTimePeriod,
        TimeStep.DAY,
        wrongFluidType
      )
      expect(result).toBeFalsy()
    })
    it('should return false because of wrong TimePeriod', () => {
      const wrongTimePeriod: TimePeriod = {
        startDate: DateTime.fromISO('2020-10-05T00:00:00.000Z', {
          zone: 'utc',
        }),
        endDate: DateTime.fromISO('2020-10-03T23:59:59.999Z', { zone: 'utc' }),
      }
      const result = consumptionValidatorService.ValidateGetGraphData(
        wrongTimePeriod,
        TimeStep.DAY,
        fluidTypes
      )
      expect(result).toBeFalsy()
    })
    it('should return false because of HALF_AN_HOUR and ELECTRICITY', () => {
      const result = consumptionValidatorService.ValidateGetGraphData(
        mockTimePeriod,
        TimeStep.HALF_AN_HOUR,
        fluidTypes
      )
      expect(result).toBeFalsy()
    })
    it('should return false because of HALF_AN_HOUR and ValidateTimePeriodLength', () => {
      fluidTypes = [0] // Only fluid without TimeStep incompatibilty
      const mockTimePeriodTooLong: TimePeriod = {
        startDate: DateTime.fromISO('2020-10-01T00:00:00.000Z', {
          zone: 'utc',
        }),
        endDate: DateTime.fromISO('2020-10-02T22:59:59.999Z', { zone: 'utc' }),
      }
      const result = consumptionValidatorService.ValidateGetGraphData(
        mockTimePeriodTooLong,
        TimeStep.HALF_AN_HOUR,
        fluidTypes
      )
      expect(result).toBeFalsy()
    })
    it('should return false because of WEEK and ValidateTimePeriodLength', () => {
      const mockTimePeriodTooLong: TimePeriod = {
        startDate: DateTime.fromISO('2020-10-01T00:00:00.000Z', {
          zone: 'utc',
        }),
        endDate: DateTime.fromISO('2020-10-09T22:59:59.999Z', { zone: 'utc' }),
      }
      const result = consumptionValidatorService.ValidateGetGraphData(
        mockTimePeriodTooLong,
        TimeStep.WEEK,
        fluidTypes
      )
      expect(result).toBeFalsy()
    })
    it('should return false because of DAY and ValidateTimePeriodLength', () => {
      const mockTimePeriodTooLong: TimePeriod = {
        startDate: DateTime.fromISO('2020-10-01T00:00:00.000Z', {
          zone: 'utc',
        }),
        endDate: DateTime.fromISO('2020-12-01T22:59:59.999Z', { zone: 'utc' }),
      }
      const result = consumptionValidatorService.ValidateGetGraphData(
        mockTimePeriodTooLong,
        TimeStep.DAY,
        fluidTypes
      )
      expect(result).toBeFalsy()
    })
    it('should return false because of MONTH and ValidateTimePeriodLength', () => {
      const mockTimePeriodTooLong: TimePeriod = {
        startDate: DateTime.fromISO('2018-10-01T00:00:00.000Z', {
          zone: 'utc',
        }),
        endDate: DateTime.fromISO('2020-12-01T22:59:59.999Z', { zone: 'utc' }),
      }
      const result = consumptionValidatorService.ValidateGetGraphData(
        mockTimePeriodTooLong,
        TimeStep.MONTH,
        fluidTypes
      )
      expect(result).toBeFalsy()
    })
    it('should return false because of YEAR and ValidateTimePeriodLength', () => {
      const mockTimePeriodTooLong: TimePeriod = {
        startDate: DateTime.fromISO('2009-10-01T00:00:00.000Z', {
          zone: 'utc',
        }),
        endDate: DateTime.fromISO('2020-12-01T22:59:59.999Z', { zone: 'utc' }),
      }
      const result = consumptionValidatorService.ValidateGetGraphData(
        mockTimePeriodTooLong,
        TimeStep.YEAR,
        fluidTypes
      )
      expect(result).toBeFalsy()
    })
  })
})
