import DateChartService from './dateChart.service'
import { TimeStep } from 'enum/timeStep.enum'
import { DateTime } from 'luxon'
import { TimePeriod } from 'models'

const localSpy = jest.spyOn(DateTime, 'local')

describe('dateChart service', () => {
  beforeEach(() => {
    localSpy.mockClear()
  })

  describe('defineTimePeriod method', () => {
    const dateChartService = new DateChartService()
    const refDate = DateTime.fromISO('2020-10-10T08:00:00.000Z', {
      zone: 'utc',
    })

    it("should return year-4 and year of reference date's year for year timestep", () => {
      const expectedStartDate = DateTime.fromISO('2016-01-01T00:00:00.000Z', {
        zone: 'utc',
      })
      const expectedEndDate = DateTime.fromISO('2020-01-01T00:00:00.000Z', {
        zone: 'utc',
      })
      const expectedTimePeriod: TimePeriod = {
        endDate: expectedEndDate,
        startDate: expectedStartDate,
      }
      const result = dateChartService.defineTimePeriod(
        refDate,
        TimeStep.YEAR,
        0
      )
      expect(result).toEqual(expectedTimePeriod)
    })

    it("should return year-9 and year-5 reference date's for year timestep", () => {
      const expectedStartDate = DateTime.fromISO('2011-01-01T00:00:00.000Z', {
        zone: 'utc',
      })
      const expectedEndDate = DateTime.fromISO('2015-01-01T00:00:00.000Z', {
        zone: 'utc',
      })
      const expectedTimePeriod: TimePeriod = {
        endDate: expectedEndDate,
        startDate: expectedStartDate,
      }
      const result = dateChartService.defineTimePeriod(
        refDate,
        TimeStep.YEAR,
        1
      )
      expect(result).toEqual(expectedTimePeriod)
    })

    it("should return year+1 and year+5 reference date's for year timestep", () => {
      const expectedStartDate = DateTime.fromISO('2021-01-01T00:00:00.000Z', {
        zone: 'utc',
      })
      const expectedEndDate = DateTime.fromISO('2025-01-01T00:00:00.000Z', {
        zone: 'utc',
      })
      const expectedTimePeriod: TimePeriod = {
        endDate: expectedEndDate,
        startDate: expectedStartDate,
      }
      const result = dateChartService.defineTimePeriod(
        refDate,
        TimeStep.YEAR,
        -1
      )
      expect(result).toEqual(expectedTimePeriod)
    })

    it("should return first and last month of reference date's year for month timestep", () => {
      const expectedStartDate = DateTime.fromISO('2020-01-01T00:00:00.000Z', {
        zone: 'utc',
      })
      const expectedEndDate = DateTime.fromISO('2020-12-01T00:00:00.000Z', {
        zone: 'utc',
      })
      const expectedTimePeriod: TimePeriod = {
        startDate: expectedStartDate,
        endDate: expectedEndDate,
      }
      const result = dateChartService.defineTimePeriod(
        refDate,
        TimeStep.MONTH,
        0
      )
      expect(result).toEqual(expectedTimePeriod)
    })

    it("should return first and last month of reference date's year -1 for month timestep", () => {
      const expectedStartDate = DateTime.fromISO('2019-01-01T00:00:00.000Z', {
        zone: 'utc',
      })
      const expectedEndDate = DateTime.fromISO('2019-12-01T00:00:00.000Z', {
        zone: 'utc',
      })
      const expectedTimePeriod: TimePeriod = {
        endDate: expectedEndDate,
        startDate: expectedStartDate,
      }
      const result = dateChartService.defineTimePeriod(
        refDate,
        TimeStep.MONTH,
        1
      )
      expect(result).toEqual(expectedTimePeriod)
    })

    it("should return first and last month of reference date's year +1 for month timestep", () => {
      const expectedStartDate = DateTime.fromISO('2021-01-01T00:00:00.000Z', {
        zone: 'utc',
      })
      const expectedEndDate = DateTime.fromISO('2021-12-01T00:00:00.000Z', {
        zone: 'utc',
      })
      const expectedTimePeriod: TimePeriod = {
        endDate: expectedEndDate,
        startDate: expectedStartDate,
      }
      const result = dateChartService.defineTimePeriod(
        refDate,
        TimeStep.MONTH,
        -1
      )
      expect(result).toEqual(expectedTimePeriod)
    })

    it("should return first and last day of reference date's month for day timestep", () => {
      const expectedStartDate = DateTime.fromISO('2020-10-01T00:00:00.000Z', {
        zone: 'utc',
      })
      const expectedEndDate = DateTime.fromISO('2020-10-31T00:00:00.000Z', {
        zone: 'utc',
      })
      const expectedTimePeriod: TimePeriod = {
        startDate: expectedStartDate,
        endDate: expectedEndDate,
      }
      const result = dateChartService.defineTimePeriod(refDate, TimeStep.DAY, 0)
      expect(result).toEqual(expectedTimePeriod)
    })

    it("should return first and last day of reference date's month -1 for day timestep", () => {
      const expectedStartDate = DateTime.fromISO('2020-09-01T00:00:00.000Z', {
        zone: 'utc',
      })
      const expectedEndDate = DateTime.fromISO('2020-09-30T00:00:00.000Z', {
        zone: 'utc',
      })
      const expectedTimePeriod: TimePeriod = {
        startDate: expectedStartDate,
        endDate: expectedEndDate,
      }
      const result = dateChartService.defineTimePeriod(refDate, TimeStep.DAY, 1)
      expect(result).toEqual(expectedTimePeriod)
    })

    it("should return first and last day of reference date's month +1 for day timestep", () => {
      const expectedStartDate = DateTime.fromISO('2020-11-01T00:00:00.000Z', {
        zone: 'utc',
      })
      const expectedEndDate = DateTime.fromISO('2020-11-30T00:00:00.000Z', {
        zone: 'utc',
      })
      const expectedTimePeriod: TimePeriod = {
        startDate: expectedStartDate,
        endDate: expectedEndDate,
      }
      const result = dateChartService.defineTimePeriod(
        refDate,
        TimeStep.DAY,
        -1
      )
      expect(result).toEqual(expectedTimePeriod)
    })

    it("should return first and last day of week at 00:00 of reference date's week for week timestep", () => {
      const expectedStartDate = DateTime.fromISO('2020-10-05T00:00:00.000Z', {
        zone: 'utc',
      })
      const expectedEndDate = DateTime.fromISO('2020-10-11T00:00:00.000Z', {
        zone: 'utc',
      })
      const expectedTimePeriod: TimePeriod = {
        endDate: expectedEndDate,
        startDate: expectedStartDate,
      }
      const result = dateChartService.defineTimePeriod(
        refDate,
        TimeStep.WEEK,
        0
      )
      expect(result).toEqual(expectedTimePeriod)
    })

    it("should return first and last day of week at 00:00 of reference date's week -1 for week timestep", () => {
      const expectedStartDate = DateTime.fromISO('2020-09-28T00:00:00.000Z', {
        zone: 'utc',
      })
      const expectedEndDate = DateTime.fromISO('2020-10-04T00:00:00.000Z', {
        zone: 'utc',
      })
      const expectedTimePeriod: TimePeriod = {
        endDate: expectedEndDate,
        startDate: expectedStartDate,
      }
      const result = dateChartService.defineTimePeriod(
        refDate,
        TimeStep.WEEK,
        1
      )
      expect(result).toEqual(expectedTimePeriod)
    })

    it("should return first and last day of week at 00:00 of reference date's week +1 for week timestep", () => {
      const expectedStartDate = DateTime.fromISO('2020-10-12T00:00:00.000Z', {
        zone: 'utc',
      })
      const expectedEndDate = DateTime.fromISO('2020-10-18T00:00:00.000Z', {
        zone: 'utc',
      })
      const expectedTimePeriod: TimePeriod = {
        endDate: expectedEndDate,
        startDate: expectedStartDate,
      }
      const result = dateChartService.defineTimePeriod(
        refDate,
        TimeStep.WEEK,
        -1
      )
      expect(result).toEqual(expectedTimePeriod)
    })

    it("should return day at 00:00 and day at 23:30 of reference date's for half-hour timestep", () => {
      const expectedStartDate = DateTime.fromISO('2020-10-10T00:00:00.000Z', {
        zone: 'utc',
      })
      const expectedEndDate = DateTime.fromISO('2020-10-10T23:30:00.000Z', {
        zone: 'utc',
      })
      const expectedTimePeriod: TimePeriod = {
        endDate: expectedEndDate,
        startDate: expectedStartDate,
      }
      const result = dateChartService.defineTimePeriod(
        refDate,
        TimeStep.HALF_AN_HOUR,
        0
      )
      expect(result).toEqual(expectedTimePeriod)
    })

    it("should return previous day at 00:00 and day at 23:30 of reference date's for half-hour timestep", () => {
      const expectedStartDate = DateTime.fromISO('2020-10-09T00:00:00.000Z', {
        zone: 'utc',
      })
      const expectedEndDate = DateTime.fromISO('2020-10-09T23:30:00.000Z', {
        zone: 'utc',
      })
      const expectedTimePeriod: TimePeriod = {
        endDate: expectedEndDate,
        startDate: expectedStartDate,
      }
      const result = dateChartService.defineTimePeriod(
        refDate,
        TimeStep.HALF_AN_HOUR,
        1
      )
      expect(result).toEqual(expectedTimePeriod)
    })

    it("should return next day at 00:00 and day at 23:30 of reference date's for half-hour timestep", () => {
      const expectedStartDate = DateTime.fromISO('2020-10-11T00:00:00.000Z', {
        zone: 'utc',
      })
      const expectedEndDate = DateTime.fromISO('2020-10-11T23:30:00.000Z', {
        zone: 'utc',
      })
      const expectedTimePeriod: TimePeriod = {
        endDate: expectedEndDate,
        startDate: expectedStartDate,
      }
      const result = dateChartService.defineTimePeriod(
        refDate,
        TimeStep.HALF_AN_HOUR,
        -1
      )
      expect(result).toEqual(expectedTimePeriod)
    })

    it('should throw error for unknown TimeStep', () => {
      try {
        dateChartService.defineTimePeriod(refDate, 4, -1)
      } catch (error) {
        expect(error).toEqual(new Error('TimeStep unknown'))
      }
    })
  })

  describe('defineFirstStepDate method', () => {
    const dateChartService = new DateChartService()
    const refDate = DateTime.fromISO('2020-10-10T08:00:00.000Z', {
      zone: 'utc',
    })

    it("should return year-4 of reference date's year for year timestep", () => {
      const expectedStartDate = DateTime.fromISO('2016-01-01T00:00:00.000Z', {
        zone: 'utc',
      })
      const result = dateChartService.defineFirstStepDate(
        refDate,
        TimeStep.YEAR,
        0
      )
      expect(result).toEqual(expectedStartDate)
    })

    it("should return year-9 reference date's for year timestep", () => {
      const expectedStartDate = DateTime.fromISO('2011-01-01T00:00:00.000Z', {
        zone: 'utc',
      })
      const result = dateChartService.defineFirstStepDate(
        refDate,
        TimeStep.YEAR,
        1
      )
      expect(result).toEqual(expectedStartDate)
    })

    it("should return year+5 reference date's for year timestep", () => {
      const expectedStartDate = DateTime.fromISO('2021-01-01T00:00:00.000Z', {
        zone: 'utc',
      })
      const result = dateChartService.defineFirstStepDate(
        refDate,
        TimeStep.YEAR,
        -1
      )
      expect(result).toEqual(expectedStartDate)
    })

    it("should return first month of reference date's year for month timestep", () => {
      const expectedStartDate = DateTime.fromISO('2020-01-01T00:00:00.000Z', {
        zone: 'utc',
      })
      const result = dateChartService.defineFirstStepDate(
        refDate,
        TimeStep.MONTH,
        0
      )
      expect(result).toEqual(expectedStartDate)
    })

    it("should return first month of reference date's year -1 for month timestep", () => {
      const expectedStartDate = DateTime.fromISO('2019-01-01T00:00:00.000Z', {
        zone: 'utc',
      })
      const result = dateChartService.defineFirstStepDate(
        refDate,
        TimeStep.MONTH,
        1
      )
      expect(result).toEqual(expectedStartDate)
    })

    it("should return first month of reference date's year +1 for month timestep", () => {
      const expectedStartDate = DateTime.fromISO('2021-01-01T00:00:00.000Z', {
        zone: 'utc',
      })
      const result = dateChartService.defineFirstStepDate(
        refDate,
        TimeStep.MONTH,
        -1
      )
      expect(result).toEqual(expectedStartDate)
    })

    it("should return first day of reference date's month for day timestep", () => {
      const expectedStartDate = DateTime.fromISO('2020-10-01T00:00:00.000Z', {
        zone: 'utc',
      })
      const result = dateChartService.defineFirstStepDate(
        refDate,
        TimeStep.DAY,
        0
      )
      expect(result).toEqual(expectedStartDate)
    })

    it("should return first day of reference date's month -1 for day timestep", () => {
      const expectedStartDate = DateTime.fromISO('2020-09-01T00:00:00.000Z', {
        zone: 'utc',
      })
      const result = dateChartService.defineFirstStepDate(
        refDate,
        TimeStep.DAY,
        1
      )
      expect(result).toEqual(expectedStartDate)
    })

    it("should return first day of reference date's month +1 for day timestep", () => {
      const expectedStartDate = DateTime.fromISO('2020-11-01T00:00:00.000Z', {
        zone: 'utc',
      })
      const result = dateChartService.defineFirstStepDate(
        refDate,
        TimeStep.DAY,
        -1
      )
      expect(result).toEqual(expectedStartDate)
    })
  })

  describe('defineLastStepDate method', () => {
    const dateChartService = new DateChartService()
    const refDate = DateTime.fromISO('2020-10-10T08:00:00.000Z', {
      zone: 'utc',
    })

    it("should return year of reference date's year for year timestep", () => {
      const expectedEndDate = DateTime.fromISO('2020-01-01T00:00:00.000Z', {
        zone: 'utc',
      })
      const result = dateChartService.defineLastStepDate(
        refDate,
        TimeStep.YEAR,
        0
      )
      expect(result).toEqual(expectedEndDate)
    })

    it("should return year-5 reference date's for year timestep", () => {
      const expectedEndDate = DateTime.fromISO('2015-01-01T00:00:00.000Z', {
        zone: 'utc',
      })
      const result = dateChartService.defineLastStepDate(
        refDate,
        TimeStep.YEAR,
        1
      )
      expect(result).toEqual(expectedEndDate)
    })

    it("should return year+5 reference date's for year timestep", () => {
      const expectedEndDate = DateTime.fromISO('2025-01-01T00:00:00.000Z', {
        zone: 'utc',
      })
      const result = dateChartService.defineLastStepDate(
        refDate,
        TimeStep.YEAR,
        -1
      )
      expect(result).toEqual(expectedEndDate)
    })

    it("should return last month of reference date's year for month timestep", () => {
      const expectedEndDate = DateTime.fromISO('2020-12-01T00:00:00.000Z', {
        zone: 'utc',
      })
      const result = dateChartService.defineLastStepDate(
        refDate,
        TimeStep.MONTH,
        0
      )
      expect(result).toEqual(expectedEndDate)
    })

    it("should return last month of reference date's year -1 for month timestep", () => {
      const expectedEndDate = DateTime.fromISO('2019-12-01T00:00:00.000Z', {
        zone: 'utc',
      })
      const result = dateChartService.defineLastStepDate(
        refDate,
        TimeStep.MONTH,
        1
      )
      expect(result).toEqual(expectedEndDate)
    })

    it("should return last month of reference date's year +1 for month timestep", () => {
      const expectedEndDate = DateTime.fromISO('2021-12-01T00:00:00.000Z', {
        zone: 'utc',
      })
      const result = dateChartService.defineLastStepDate(
        refDate,
        TimeStep.MONTH,
        -1
      )
      expect(result).toEqual(expectedEndDate)
    })

    it("should return last day of reference date's month for day timestep", () => {
      const expectedEndDate = DateTime.fromISO('2020-10-31T00:00:00.000Z', {
        zone: 'utc',
      })
      const result = dateChartService.defineLastStepDate(
        refDate,
        TimeStep.DAY,
        0
      )
      expect(result).toEqual(expectedEndDate)
    })

    it("should return last day of reference date's month -1 for day timestep", () => {
      const expectedEndDate = DateTime.fromISO('2020-09-30T00:00:00.000Z', {
        zone: 'utc',
      })
      const result = dateChartService.defineLastStepDate(
        refDate,
        TimeStep.DAY,
        1
      )
      expect(result).toEqual(expectedEndDate)
    })

    it("should return last day of reference date's month +1 for day timestep", () => {
      const expectedEndDate = DateTime.fromISO('2020-11-30T00:00:00.000Z', {
        zone: 'utc',
      })
      const result = dateChartService.defineLastStepDate(
        refDate,
        TimeStep.DAY,
        -1
      )
      expect(result).toEqual(expectedEndDate)
    })
  })

  describe('compareStepDate method', () => {
    const dateChartService = new DateChartService()

    it('should return true for year comparaison', () => {
      const firstDate = DateTime.fromISO('2020-10-31T00:00:00.000Z', {
        zone: 'utc',
      })
      const secondDate = DateTime.fromISO('2020-12-31T11:11:11.111Z', {
        zone: 'utc',
      })
      const result = dateChartService.compareStepDate(
        TimeStep.YEAR,
        firstDate,
        secondDate
      )
      expect(result).toBe(true)
    })

    it('should return false for year comparaison', () => {
      const firstDate = DateTime.fromISO('2020-10-31T00:00:00.000Z', {
        zone: 'utc',
      })
      const secondDate = DateTime.fromISO('2021-10-30T00:00:00.000Z', {
        zone: 'utc',
      })
      const result = dateChartService.compareStepDate(
        TimeStep.YEAR,
        firstDate,
        secondDate
      )
      expect(result).toBe(false)
    })

    it('should return true for month comparaison', () => {
      const firstDate = DateTime.fromISO('2020-10-31T00:00:00.000Z', {
        zone: 'utc',
      })
      const secondDate = DateTime.fromISO('2020-10-30T11:11:11.111Z', {
        zone: 'utc',
      })
      const result = dateChartService.compareStepDate(
        TimeStep.MONTH,
        firstDate,
        secondDate
      )
      expect(result).toBe(true)
    })

    it('should return false for month comparaison', () => {
      const firstDate = DateTime.fromISO('2020-10-31T00:00:00.000Z', {
        zone: 'utc',
      })
      const secondDate = DateTime.fromISO('2020-09-31T00:00:00.000Z', {
        zone: 'utc',
      })
      const result = dateChartService.compareStepDate(
        TimeStep.MONTH,
        firstDate,
        secondDate
      )
      expect(result).toBe(false)
    })

    it('should return true for day comparaison', () => {
      const firstDate = DateTime.fromISO('2020-10-31T00:00:00.000Z', {
        zone: 'utc',
      })
      const secondDate = DateTime.fromISO('2020-10-31T11:11:11.111Z', {
        zone: 'utc',
      })
      const result = dateChartService.compareStepDate(
        TimeStep.DAY,
        firstDate,
        secondDate
      )
      expect(result).toBe(true)
    })

    it('should return false for day comparaison', () => {
      const firstDate = DateTime.fromISO('2020-10-31T00:00:00.000Z', {
        zone: 'utc',
      })
      const secondDate = DateTime.fromISO('2020-10-30T00:00:00.000Z', {
        zone: 'utc',
      })
      const result = dateChartService.compareStepDate(
        TimeStep.DAY,
        firstDate,
        secondDate
      )
      expect(result).toBe(false)
    })

    it('should return true for week comparaison', () => {
      const firstDate = DateTime.fromISO('2020-10-31T00:00:00.000Z', {
        zone: 'utc',
      })
      const secondDate = DateTime.fromISO('2020-10-31T11:11:11.111Z', {
        zone: 'utc',
      })
      const result = dateChartService.compareStepDate(
        TimeStep.WEEK,
        firstDate,
        secondDate
      )
      expect(result).toBe(true)
    })

    it('should return false for week comparaison', () => {
      const firstDate = DateTime.fromISO('2020-10-31T00:00:00.000Z', {
        zone: 'utc',
      })
      const secondDate = DateTime.fromISO('2020-10-30T00:00:00.000Z', {
        zone: 'utc',
      })
      const result = dateChartService.compareStepDate(
        TimeStep.WEEK,
        firstDate,
        secondDate
      )
      expect(result).toBe(false)
    })

    it('should return true for half-hour comparaison', () => {
      const firstDate = DateTime.fromISO('2020-10-31T00:00:00.000Z', {
        zone: 'utc',
      })
      const secondDate = DateTime.fromISO('2020-10-31T00:00:11.111Z', {
        zone: 'utc',
      })
      const result = dateChartService.compareStepDate(
        TimeStep.HALF_AN_HOUR,
        firstDate,
        secondDate
      )
      expect(result).toBe(true)
    })

    it('should return false for half-hour comparaison', () => {
      const firstDate = DateTime.fromISO('2020-10-31T00:00:00.000Z', {
        zone: 'utc',
      })
      const secondDate = DateTime.fromISO('2020-10-31T00:30:00.000Z', {
        zone: 'utc',
      })
      const result = dateChartService.compareStepDate(
        TimeStep.HALF_AN_HOUR,
        firstDate,
        secondDate
      )
      expect(result).toBe(false)
    })

    it('should throw error for unknown TimeStep', () => {
      const firstDate = DateTime.fromISO('2020-10-31T00:00:00.000Z', {
        zone: 'utc',
      })
      const secondDate = DateTime.fromISO('2020-10-31T00:30:00.000Z', {
        zone: 'utc',
      })
      try {
        dateChartService.compareStepDate(4, firstDate, secondDate)
      } catch (error) {
        expect(error).toEqual(new Error('TimeStep unknown'))
      }
    })
  })

  describe('defineIncrementForPreviousIndex', () => {
    const dateChartService = new DateChartService()
    localSpy.mockReturnValue(
      DateTime.fromISO('2020-10-10T08:08:08.008Z', { zone: 'utc' })
    )

    it('should return -4 for year, 2023 on index -1', () => {
      const selectedDate: DateTime = DateTime.fromISO(
        '2023-10-31T00:00:00.000Z',
        {
          zone: 'utc',
        }
      )
      const result = dateChartService.defineIncrementForPreviousIndex(
        TimeStep.YEAR,
        selectedDate,
        -1
      )
      expect(result).toBe(-3)
    })

    it('should return -1 for year, 2016 on index 0', () => {
      const selectedDate: DateTime = DateTime.fromISO(
        '2016-10-31T00:00:00.000Z',
        {
          zone: 'utc',
        }
      )
      const result = dateChartService.defineIncrementForPreviousIndex(
        TimeStep.YEAR,
        selectedDate,
        0
      )
      expect(result).toBe(-1)
    })

    it('should return -3 for year, 2013 on index 1', () => {
      const selectedDate: DateTime = DateTime.fromISO(
        '2013-10-31T00:00:00.000Z',
        {
          zone: 'utc',
        }
      )
      const result = dateChartService.defineIncrementForPreviousIndex(
        TimeStep.YEAR,
        selectedDate,
        1
      )
      expect(result).toBe(-3)
    })

    it('should return -2 for month, 02-2021 on index -1', () => {
      const selectedDate: DateTime = DateTime.fromISO(
        '2021-02-01T00:00:00.000Z',
        {
          zone: 'utc',
        }
      )
      const result = dateChartService.defineIncrementForPreviousIndex(
        TimeStep.MONTH,
        selectedDate,
        -1
      )
      expect(result).toBe(-2)
    })

    it('should return -23 for day, 23-06-2020 on index 0', () => {
      const selectedDate: DateTime = DateTime.fromISO(
        '2020-06-23T00:00:00.000Z',
        {
          zone: 'utc',
        }
      )
      const result = dateChartService.defineIncrementForPreviousIndex(
        TimeStep.DAY,
        selectedDate,
        0
      )
      expect(result).toBe(-23)
    })

    it('should return -2 for week, 06-10-2020 on index 0', () => {
      const selectedDate: DateTime = DateTime.fromISO(
        '2020-10-06T00:00:00.000Z',
        {
          zone: 'utc',
        }
      )
      const result = dateChartService.defineIncrementForPreviousIndex(
        TimeStep.WEEK,
        selectedDate,
        0
      )
      expect(result).toBe(-2)
    })

    it('should return -19 for half an hour, 09-10-2020 at 9.00 on index 0', () => {
      const selectedDate: DateTime = DateTime.fromISO(
        '2020-10-09T09:00:00.000Z',
        {
          zone: 'utc',
        }
      )
      const result = dateChartService.defineIncrementForPreviousIndex(
        TimeStep.HALF_AN_HOUR,
        selectedDate,
        0
      )
      expect(result).toBe(-19)
    })

    it('should return -20 for half an hour, 09-10-2020 at 9.30 on index 0', () => {
      const selectedDate: DateTime = DateTime.fromISO(
        '2020-10-09T09:30:00.000Z',
        {
          zone: 'utc',
        }
      )
      const result = dateChartService.defineIncrementForPreviousIndex(
        TimeStep.HALF_AN_HOUR,
        selectedDate,
        0
      )
      expect(result).toBe(-20)
    })

    it('should return 0 for unkonwn time step', () => {
      const selectedDate: DateTime = DateTime.fromISO(
        '2020-10-09T09:30:00.000Z',
        {
          zone: 'utc',
        }
      )
      const result = dateChartService.defineIncrementForPreviousIndex(
        0,
        selectedDate,
        0
      )
      expect(result).toBe(0)
    })
  })

  describe('defineIncrementForNextIndex', () => {
    const dateChartService = new DateChartService()
    localSpy.mockReturnValue(
      DateTime.fromISO('2020-10-10T08:08:08.008Z', { zone: 'utc' })
    )

    it('should return 7 for year, 2023 on index -1', () => {
      const selectedDate: DateTime = DateTime.fromISO(
        '2023-10-31T00:00:00.000Z',
        {
          zone: 'utc',
        }
      )
      const result = dateChartService.defineIncrementForNextIndex(
        TimeStep.YEAR,
        selectedDate,
        -1
      )
      expect(result).toBe(7)
    })

    it('should return 9 for year, 2016 on index 0', () => {
      const selectedDate: DateTime = DateTime.fromISO(
        '2016-10-31T00:00:00.000Z',
        {
          zone: 'utc',
        }
      )
      const result = dateChartService.defineIncrementForNextIndex(
        TimeStep.YEAR,
        selectedDate,
        0
      )
      expect(result).toBe(9)
    })

    it('should return 7 for year, 2013 on index 1', () => {
      const selectedDate: DateTime = DateTime.fromISO(
        '2013-10-31T00:00:00.000Z',
        {
          zone: 'utc',
        }
      )
      const result = dateChartService.defineIncrementForNextIndex(
        TimeStep.YEAR,
        selectedDate,
        1
      )
      expect(result).toBe(7)
    })

    it('should return 22 for month, 02-2021 on index 0', () => {
      const selectedDate: DateTime = DateTime.fromISO(
        '2021-02-01T00:00:00.000Z',
        {
          zone: 'utc',
        }
      )
      const result = dateChartService.defineIncrementForNextIndex(
        TimeStep.MONTH,
        selectedDate,
        0
      )
      expect(result).toBe(22)
    })

    it('should return 51 for day, 10-10-2020 on index 0', () => {
      const selectedDate: DateTime = DateTime.fromISO(
        '2020-10-10T00:00:00.000Z',
        {
          zone: 'utc',
        }
      )
      const result = dateChartService.defineIncrementForNextIndex(
        TimeStep.DAY,
        selectedDate,
        0
      )
      expect(result).toBe(51)
    })

    it('should return 8 for week, 10-10-2020 on index 0', () => {
      const selectedDate: DateTime = DateTime.fromISO(
        '2020-10-10T00:00:00.000Z',
        {
          zone: 'utc',
        }
      )
      const result = dateChartService.defineIncrementForNextIndex(
        TimeStep.WEEK,
        selectedDate,
        0
      )
      expect(result).toBe(8)
    })

    it('should return 77 for half an hour, 09-10-2020 at 9.00 on index 0', () => {
      const selectedDate: DateTime = DateTime.fromISO(
        '2020-10-09T09:00:00.000Z',
        {
          zone: 'utc',
        }
      )
      const result = dateChartService.defineIncrementForNextIndex(
        TimeStep.HALF_AN_HOUR,
        selectedDate,
        0
      )
      expect(result).toBe(77)
    })

    it('should return 76 for half an hour, 09-10-2020 at 9.30 on index 0', () => {
      const selectedDate: DateTime = DateTime.fromISO(
        '2020-10-09T09:30:00.000Z',
        {
          zone: 'utc',
        }
      )
      const result = dateChartService.defineIncrementForNextIndex(
        TimeStep.HALF_AN_HOUR,
        selectedDate,
        0
      )
      expect(result).toBe(76)
    })

    it('should return 0 for unkonwn time step', () => {
      const selectedDate: DateTime = DateTime.fromISO(
        '2020-10-09T09:30:00.000Z',
        {
          zone: 'utc',
        }
      )
      const result = dateChartService.defineIncrementForNextIndex(
        0,
        selectedDate,
        0
      )
      expect(result).toBe(0)
    })
  })

  describe('incrementeDate method', () => {
    const dateChartService = new DateChartService()
    const selectedDate: DateTime = DateTime.fromISO(
      '2020-10-31T00:00:00.000Z',
      {
        zone: 'utc',
      }
    )

    it('should return incremented year', () => {
      const expectedDate: DateTime = DateTime.fromISO(
        '2021-10-31T00:00:00.000Z',
        {
          zone: 'utc',
        }
      )
      const result = dateChartService.incrementeDate(
        TimeStep.YEAR,
        selectedDate,
        1
      )
      expect(result).toEqual(expectedDate)
    })

    it('should return incremented month', () => {
      const expectedDate: DateTime = DateTime.fromISO(
        '2020-11-30T00:00:00.000Z',
        {
          zone: 'utc',
        }
      )
      const result = dateChartService.incrementeDate(
        TimeStep.MONTH,
        selectedDate,
        1
      )
      expect(result).toEqual(expectedDate)
    })

    it('should return incremented day', () => {
      const expectedDate: DateTime = DateTime.fromISO(
        '2020-11-01T00:00:00.000Z',
        {
          zone: 'utc',
        }
      )
      const result = dateChartService.incrementeDate(
        TimeStep.DAY,
        selectedDate,
        1
      )
      expect(result).toEqual(expectedDate)
    })

    it('should return incremented day in week', () => {
      const expectedDate: DateTime = DateTime.fromISO(
        '2020-11-01T00:00:00.000Z',
        {
          zone: 'utc',
        }
      )
      const result = dateChartService.incrementeDate(
        TimeStep.WEEK,
        selectedDate,
        1
      )
      expect(result).toEqual(expectedDate)
    })

    it('should return incremented half an hour in week', () => {
      const expectedDate: DateTime = DateTime.fromISO(
        '2020-10-31T00:30:00.000Z',
        {
          zone: 'utc',
        }
      )
      const result = dateChartService.incrementeDate(
        TimeStep.HALF_AN_HOUR,
        selectedDate,
        1
      )
      expect(result).toEqual(expectedDate)
    })

    it('should return selected date for unknown time step', () => {
      const result = dateChartService.incrementeDate(0, selectedDate, 1)
      expect(result).toEqual(selectedDate)
    })
  })

  describe('defineDateIndex method', () => {
    const dateChartService = new DateChartService()
    localSpy.mockReturnValue(
      DateTime.fromISO('2020-10-10T08:08:08.008Z', { zone: 'utc' })
    )

    it('should return index 0 for year, 2016', () => {
      const selectedDate: DateTime = DateTime.fromISO(
        '2016-01-01T00:00:00.000Z',
        {
          zone: 'utc',
        }
      )
      const result = dateChartService.defineDateIndex(
        TimeStep.YEAR,
        selectedDate
      )
      expect(result).toEqual(0)
    })
    it('should return index 1 for year, 2012', () => {
      const selectedDate: DateTime = DateTime.fromISO(
        '2012-01-01T00:00:00.000Z',
        {
          zone: 'utc',
        }
      )
      const result = dateChartService.defineDateIndex(
        TimeStep.YEAR,
        selectedDate
      )
      expect(result).toEqual(1)
    })
    it('should return index -1 for year, 2024', () => {
      const selectedDate: DateTime = DateTime.fromISO(
        '2024-01-01T00:00:00.000Z',
        {
          zone: 'utc',
        }
      )
      const result = dateChartService.defineDateIndex(
        TimeStep.YEAR,
        selectedDate
      )
      expect(result).toEqual(-1)
    })

    it('should return index 0 for month, 05-2020', () => {
      const selectedDate: DateTime = DateTime.fromISO(
        '2020-05-01T00:00:00.000Z',
        {
          zone: 'utc',
        }
      )
      const result = dateChartService.defineDateIndex(
        TimeStep.MONTH,
        selectedDate
      )
      expect(result).toEqual(0)
    })
    it('should return index 1 for month, 05-2019', () => {
      const selectedDate: DateTime = DateTime.fromISO(
        '2019-05-01T00:00:00.000Z',
        {
          zone: 'utc',
        }
      )
      const result = dateChartService.defineDateIndex(
        TimeStep.MONTH,
        selectedDate
      )
      expect(result).toEqual(1)
    })
    it('should return index -1 for month, 05-2021', () => {
      const selectedDate: DateTime = DateTime.fromISO(
        '2021-05-01T00:00:00.000Z',
        {
          zone: 'utc',
        }
      )
      const result = dateChartService.defineDateIndex(
        TimeStep.MONTH,
        selectedDate
      )
      expect(result).toEqual(-1)
    })

    it('should return index 0 for day, 02-10-2020', () => {
      const selectedDate: DateTime = DateTime.fromISO(
        '2020-10-02T00:00:00.000Z',
        {
          zone: 'utc',
        }
      )
      const result = dateChartService.defineDateIndex(
        TimeStep.DAY,
        selectedDate
      )
      expect(result).toEqual(0)
    })
    it('should return index 1 for day, 02-09-2020', () => {
      const selectedDate: DateTime = DateTime.fromISO(
        '2020-09-02T00:00:00.000Z',
        {
          zone: 'utc',
        }
      )
      const result = dateChartService.defineDateIndex(
        TimeStep.DAY,
        selectedDate
      )
      expect(result).toEqual(1)
    })
    it('should return index -1 for day, 02-11-2020', () => {
      const selectedDate: DateTime = DateTime.fromISO(
        '2020-11-02T00:00:00.000Z',
        {
          zone: 'utc',
        }
      )
      const result = dateChartService.defineDateIndex(
        TimeStep.DAY,
        selectedDate
      )
      expect(result).toEqual(-1)
    })

    it('should return index 0 for day in week, 05-10-2020', () => {
      const selectedDate: DateTime = DateTime.fromISO(
        '2020-10-05T00:00:00.000Z',
        {
          zone: 'utc',
        }
      )
      const result = dateChartService.defineDateIndex(
        TimeStep.WEEK,
        selectedDate
      )
      expect(result).toEqual(0)
    })
    it('should return index 1 for day in week, 04-10-2020', () => {
      const selectedDate: DateTime = DateTime.fromISO(
        '2020-10-04T00:00:00.000Z',
        {
          zone: 'utc',
        }
      )
      const result = dateChartService.defineDateIndex(
        TimeStep.WEEK,
        selectedDate
      )
      expect(result).toEqual(1)
    })
    it('should return index -1 for day in week, 12-10-2020', () => {
      const selectedDate: DateTime = DateTime.fromISO(
        '2020-10-12T00:00:00.000Z',
        {
          zone: 'utc',
        }
      )
      const result = dateChartService.defineDateIndex(
        TimeStep.WEEK,
        selectedDate
      )
      expect(result).toEqual(-1)
    })

    it('should return index 0 for half an hour, 10-10-2020 at 9.00', () => {
      const selectedDate: DateTime = DateTime.fromISO(
        '2020-10-10T09:00:00.000Z',
        {
          zone: 'utc',
        }
      )
      const result = dateChartService.defineDateIndex(
        TimeStep.HALF_AN_HOUR,
        selectedDate
      )
      expect(result).toEqual(0)
    })
    it('should return index 1 for half an hour, 09-10-2020 at 9.00', () => {
      const selectedDate: DateTime = DateTime.fromISO(
        '2020-10-09T09:00:00.000Z',
        {
          zone: 'utc',
        }
      )
      const result = dateChartService.defineDateIndex(
        TimeStep.HALF_AN_HOUR,
        selectedDate
      )
      expect(result).toEqual(1)
    })
    it('should return index -1 for half an hour, 11-10-2020 at 9.00', () => {
      const selectedDate: DateTime = DateTime.fromISO(
        '2020-10-11T09:00:00.000Z',
        {
          zone: 'utc',
        }
      )
      const result = dateChartService.defineDateIndex(
        TimeStep.HALF_AN_HOUR,
        selectedDate
      )
      expect(result).toEqual(-1)
    })

    it('should return 0 for unknown time step', () => {
      const selectedDate: DateTime = DateTime.fromISO(
        '2020-10-09T09:00:00.000Z',
        {
          zone: 'utc',
        }
      )
      const result = dateChartService.defineDateIndex(0, selectedDate)
      expect(result).toEqual(0)
    })
  })
})
