import { QueryResult } from 'cozy-client'
import { Ecogesture } from 'models'
import EcogestureService from './ecogesture.service'
import mockClient from '../../tests/__mocks__/client'
import {
  BoilerEcogesture,
  BoilerEcogestureFalse,
  ecogesturesData,
  ecogesturesECSData,
  ecogesturesHeatingData,
} from '../../tests/__mocks__/ecogesturesData.mock'
import { IndividualOrCollective, WarmingType } from 'enum/profileType.enum'
import { mockProfileEcogesture } from '../../tests/__mocks__/profileEcogesture.mock'
import { ProfileEcogesture } from 'models/profileEcogesture.model'
import { EquipmentType } from 'enum/ecogesture.enum'

describe('Ecogesture service', () => {
  const ecogestureService = new EcogestureService(mockClient)
  describe('getAllEcogestures', () => {
    it('should return all ecogestures', async () => {
      const mockQueryResult: QueryResult<Ecogesture[]> = {
        data: ecogesturesData,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValueOnce(mockQueryResult)
      const result = await ecogestureService.getAllEcogestures()
      expect(result).toEqual(ecogesturesData)
    })
    it('should return empty array when no ecogestures stored', async () => {
      const mockQueryResult: QueryResult<Ecogesture[]> = {
        data: [],
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValueOnce(mockQueryResult)
      const result = await ecogestureService.getAllEcogestures()
      expect(result).toEqual([])
    })
  })

  describe('deleteAllEcogestures', () => {
    it('should return true when 3 ecogestures stored', async () => {
      const mockQueryResult: QueryResult<Ecogesture[]> = {
        data: ecogesturesData,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValueOnce(mockQueryResult)
      const result = await ecogestureService.deleteAllEcogestures()
      expect(mockClient.destroy).toBeCalledTimes(3)
      expect(result).toBe(true)
    })
    it('should return true when no ecogestures stored', async () => {
      const mockQueryResult: QueryResult<Ecogesture[]> = {
        data: [],
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValueOnce(mockQueryResult)
      const result = await ecogestureService.deleteAllEcogestures()
      expect(result).toBe(true)
    })
    it('should throw exception when error happened on deletion', async () => {
      const mockQueryResult: QueryResult<Ecogesture[]> = {
        data: ecogesturesData,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.destroy.mockRejectedValue(new Error())
      mockClient.query.mockResolvedValueOnce(mockQueryResult)
      await expect(ecogestureService.deleteAllEcogestures()).rejects.toThrow(
        new Error()
      )
    })
  })
  describe('reinitAllEcogestures', () => {
    it('should return true when 3 ecogestures stored', async () => {
      const mockQueryResult: QueryResult<Ecogesture[]> = {
        data: ecogesturesData,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValueOnce(mockQueryResult)
      const result = await ecogestureService.reinitAllEcogestures()
      expect(mockClient.save).toBeCalledTimes(3)
      expect(result).toBe(true)
    })
    it('should return true when no ecogestures stored', async () => {
      const mockQueryResult: QueryResult<Ecogesture[]> = {
        data: [],
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValueOnce(mockQueryResult)
      const result = await ecogestureService.reinitAllEcogestures()
      expect(result).toBe(true)
    })
    it('should throw exception when error happened on reinit', async () => {
      const mockQueryResult: QueryResult<Ecogesture[]> = {
        data: ecogesturesData,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.save.mockRejectedValue(new Error())
      mockClient.query.mockResolvedValueOnce(mockQueryResult)
      await expect(ecogestureService.reinitAllEcogestures()).rejects.toThrow(
        new Error()
      )
    })
  })

  describe('filterByUsage', () => {
    it('should return ecogesture list including ECS ecogestures', async () => {
      const mockEcogestureList: Ecogesture[] = ecogesturesECSData
      const mockProfileEcogesture1: ProfileEcogesture = {
        ...mockProfileEcogesture,
        hotWater: IndividualOrCollective.INDIVIDUAL,
      }
      const result = ecogestureService.filterByUsage(
        mockEcogestureList,
        mockProfileEcogesture1
      )
      expect(result.includes(ecogesturesECSData[0])).toBeTruthy()
    })
    it('should return ecogesture list excluding ECS ecogestures', async () => {
      const mockEcogestureList: Ecogesture[] = ecogesturesECSData
      const mockProfileEcogesture1: ProfileEcogesture = {
        ...mockProfileEcogesture,
        hotWater: IndividualOrCollective.COLLECTIVE,
      }
      const result = ecogestureService.filterByUsage(
        mockEcogestureList,
        mockProfileEcogesture1
      )
      expect(result.includes(ecogesturesECSData[1])).toBeFalsy()
    })
    it('should return ecogesture list including HEATING ecogestures', async () => {
      const mockEcogestureList: Ecogesture[] = ecogesturesHeatingData
      const mockProfileEcogesture2: ProfileEcogesture = {
        ...mockProfileEcogesture,
        heating: IndividualOrCollective.INDIVIDUAL,
        warmingFluid: WarmingType.ELECTRICITY,
      }
      const result = ecogestureService.filterByUsage(
        mockEcogestureList,
        mockProfileEcogesture2
      )
      expect(result.includes(ecogesturesHeatingData[0])).toBeTruthy()
    })
    it('should return ecogesture list excluding HEATING ecogestures', async () => {
      const mockEcogestureList: Ecogesture[] = ecogesturesHeatingData
      const mockProfileEcogesture2: ProfileEcogesture = {
        ...mockProfileEcogesture,
        heating: IndividualOrCollective.COLLECTIVE,
      }
      const result = ecogestureService.filterByUsage(
        mockEcogestureList,
        mockProfileEcogesture2
      )
      expect(result.includes(ecogesturesHeatingData[0])).toBeFalsy()
    })
  })
  describe('filterByEquipment', () => {
    it('should return ecogesture list including BOILER equipment and equipment veriication to true', async () => {
      const mockProfileEcogestureBOILER: ProfileEcogesture = {
        ...mockProfileEcogesture,
        equipments: [EquipmentType.BOILER],
      }
      const result = ecogestureService.filterByEquipment(
        BoilerEcogesture,
        mockProfileEcogestureBOILER
      )
      expect(result.includes(BoilerEcogesture[0])).toBeTruthy()
    })
    it('should return ecogesture list excluding BOILER equipment and equipment veriication to true', async () => {
      const mockProfileEcogestureBOILER: ProfileEcogesture = {
        ...mockProfileEcogesture,
      }
      const result = ecogestureService.filterByEquipment(
        BoilerEcogesture,
        mockProfileEcogestureBOILER
      )
      expect(result.includes(BoilerEcogesture[0])).toBeFalsy()
    })
    it('should return ecogesture list including BOILER equipment with equipment veriication to false but equipment to BOILER', async () => {
      const mockProfileEcogestureBOILER: ProfileEcogesture = {
        ...mockProfileEcogesture,
        equipments: [EquipmentType.BOILER],
      }
      const result = ecogestureService.filterByEquipment(
        BoilerEcogestureFalse,
        mockProfileEcogestureBOILER
      )
      expect(result.includes(BoilerEcogestureFalse[0])).toBeTruthy()
    })
  })
  describe('getEcogestureListByProfile', () => {
    it('should return ecogesture list according to profile ecogesture, sorted and filtered', async () => {
      const mockProfileEcogestureFull: ProfileEcogesture = {
        ...mockProfileEcogesture,
        equipments: [EquipmentType.WASHING_MACHINE, EquipmentType.DISHWASHER],
      }
      const mockQueryResult: QueryResult<Ecogesture[]> = {
        data: ecogesturesData,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValueOnce(mockQueryResult)

      const result = await ecogestureService.getEcogestureListByProfile(
        mockProfileEcogestureFull
      )
      expect(result.length).toBe(2)
      expect(result[0]).toBe(ecogesturesData[0])
    })
  })
  describe('getEcogesturesByIds', () => {
    it('Should return corresponding ecogestures', async () => {
      const mockQueryResult: QueryResult<Ecogesture[]> = {
        data: ecogesturesECSData,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValueOnce(mockQueryResult)

      const result = await ecogestureService.getEcogesturesByIds([
        'ECOGESTURE0001',
        'ECOGESTURE0002',
        'ECOGESTURE0013',
      ])
      expect(result.length).toBe(3)
      expect(result).toBe(ecogesturesECSData)
    })
  })
})
