import { QueryResult } from 'cozy-client'
import { DataloadState } from 'enum/dataload.enum'
import { FluidType } from 'enum/fluid.enum'
import { TimeStep } from 'enum/timeStep.enum'
import { DateTime } from 'luxon'
import { Dataload, DataloadEntity } from 'models'
import mockClient from '../../tests/__mocks__/client'
import { loadDayData } from '../../tests/__mocks__/loadDayData.mock'
import { loadMinuteData } from '../../tests/__mocks__/loadMinuteData.mock'
import { loadMonthData } from '../../tests/__mocks__/loadMonthData.mock'
import { loadYearData } from '../../tests/__mocks__/loadYearData.mock'
import QueryRunner from './queryRunner.service'

describe('queryRunner service', () => {
  const queryRunner = new QueryRunner(mockClient)

  describe('fetchFluidData method', () => {
    it('should return the data of the elec fluid and year time step', async () => {
      const mockTimePeriod = {
        startDate: DateTime.fromISO('2018-01-01T00:00:00.000Z', {
          zone: 'utc',
        }),
        endDate: DateTime.fromISO('2020-11-30T23:59:59.999Z', {
          zone: 'utc',
        }),
      }
      const expectedResult: Dataload[] = [
        {
          date: DateTime.fromISO('2018-01-01T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 125.25,
          state: DataloadState.VALID,
          valueDetail: null,
        },
        {
          date: DateTime.fromISO('2019-01-01T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 220.5,
          state: DataloadState.VALID,
          valueDetail: null,
        },
        {
          date: DateTime.fromISO('2020-01-01T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 130.33,
          state: DataloadState.VALID,
          valueDetail: null,
        },
      ]
      const mockQueryResult: QueryResult<DataloadEntity[]> = {
        data: loadYearData,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValue(mockQueryResult)
      const result: Dataload[] | null = await queryRunner.fetchFluidData(
        mockTimePeriod,
        TimeStep.YEAR,
        FluidType.ELECTRICITY
      )
      expect(result).toEqual(expectedResult)
    })

    it('should return the data of the elec fluid and month time step', async () => {
      const mockTimePeriod = {
        startDate: DateTime.fromISO('2020-01-01T00:00:00.000Z', {
          zone: 'utc',
        }),
        endDate: DateTime.fromISO('2020-11-30T23:59:59.999Z', {
          zone: 'utc',
        }),
      }
      const expectedResult: Dataload[] = [
        {
          date: DateTime.fromISO('2020-09-01T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 125.25,
          state: DataloadState.VALID,
          valueDetail: null,
        },
        {
          date: DateTime.fromISO('2020-10-01T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 220.5,
          state: DataloadState.VALID,
          valueDetail: null,
        },
        {
          date: DateTime.fromISO('2020-11-01T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 130.33,
          state: DataloadState.VALID,
          valueDetail: null,
        },
      ]
      const mockQueryResult: QueryResult<DataloadEntity[]> = {
        data: loadMonthData,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValue(mockQueryResult)
      const result: Dataload[] | null = await queryRunner.fetchFluidData(
        mockTimePeriod,
        TimeStep.MONTH,
        FluidType.ELECTRICITY
      )
      expect(result).toEqual(expectedResult)
    })

    it('should return the data of the elec fluid and day time step', async () => {
      const mockTimePeriod = {
        startDate: DateTime.fromISO('2020-11-01T00:00:00.000Z', {
          zone: 'utc',
        }),
        endDate: DateTime.fromISO('2020-11-30T23:59:59.999Z', {
          zone: 'utc',
        }),
      }
      const expectedResult: Dataload[] = [
        {
          date: DateTime.fromISO('2020-11-01T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 25.25,
          price: 0.12,
          state: DataloadState.VALID,
          valueDetail: null,
        },
        {
          date: DateTime.fromISO('2020-11-02T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 20.5,
          price: 0.14,
          state: DataloadState.VALID,
          valueDetail: null,
        },
        {
          date: DateTime.fromISO('2020-11-03T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 30.33,
          price: 0.18,
          state: DataloadState.VALID,
          valueDetail: null,
        },
        {
          date: DateTime.fromISO('2020-11-04T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 1.22,
          price: 0.16,
          state: DataloadState.VALID,
          valueDetail: null,
        },
      ]
      const mockQueryResult: QueryResult<DataloadEntity[]> = {
        data: loadDayData,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValue(mockQueryResult)
      const result: Dataload[] | null = await queryRunner.fetchFluidData(
        mockTimePeriod,
        TimeStep.DAY,
        FluidType.ELECTRICITY
      )
      expect(result).toEqual(expectedResult)
    })

    it('should return the data of the elec fluid and week time step', async () => {
      const mockTimePeriod = {
        startDate: DateTime.fromISO('2020-11-02T00:00:00.000Z', {
          zone: 'utc',
        }),
        endDate: DateTime.fromISO('2020-11-08T23:59:59.999Z', {
          zone: 'utc',
        }),
      }
      const expectedResult: Dataload[] = [
        {
          date: DateTime.fromISO('2020-11-02T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 20.5,
          price: 0.14,
          state: DataloadState.VALID,
          valueDetail: null,
        },
        {
          date: DateTime.fromISO('2020-11-03T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 30.33,
          price: 0.18,
          state: DataloadState.VALID,
          valueDetail: null,
        },
        {
          date: DateTime.fromISO('2020-11-04T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 1.22,
          price: 0.16,
          state: DataloadState.VALID,
          valueDetail: null,
        },
      ]
      const mockQueryResult: QueryResult<DataloadEntity[]> = {
        data: loadDayData,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValue(mockQueryResult)
      const result: Dataload[] | null = await queryRunner.fetchFluidData(
        mockTimePeriod,
        TimeStep.WEEK,
        FluidType.ELECTRICITY
      )
      expect(result).toEqual(expectedResult)
    })

    it('should return the data of the elec fluid and half an hour time step', async () => {
      const mockTimePeriod = {
        startDate: DateTime.fromISO('2020-11-02T00:00:00.000Z', {
          zone: 'utc',
        }),
        endDate: DateTime.fromISO('2020-11-02T23:59:59.999Z', {
          zone: 'utc',
        }),
      }
      const expectedResult: Dataload[] = [
        {
          date: DateTime.fromISO('2020-11-02T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 1.33,
          state: DataloadState.VALID,
          valueDetail: null,
        },
        {
          date: DateTime.fromISO('2020-11-02T00:30:00.000Z', {
            zone: 'utc',
          }),
          value: 3.22,
          state: DataloadState.VALID,
          valueDetail: null,
        },
        {
          date: DateTime.fromISO('2020-11-02T01:00:00.000Z', {
            zone: 'utc',
          }),
          value: 7.82,
          state: DataloadState.VALID,
          valueDetail: null,
        },
        {
          date: DateTime.fromISO('2020-11-02T01:30:00.000Z', {
            zone: 'utc',
          }),
          value: 1.23,
          state: DataloadState.VALID,
          valueDetail: null,
        },
      ]
      const filteredData = [...loadMinuteData].filter(
        data => data.year === 2020 && data.month === 11 && data.day === 2
      )
      const mockQueryResult: QueryResult<DataloadEntity[]> = {
        data: filteredData,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValue(mockQueryResult)
      const result: Dataload[] | null = await queryRunner.fetchFluidData(
        mockTimePeriod,
        TimeStep.HALF_AN_HOUR,
        FluidType.ELECTRICITY
      )
      expect(result).toEqual(expectedResult)
    })

    it('should return the data of the water fluid and year time step', async () => {
      const mockTimePeriod = {
        startDate: DateTime.fromISO('2018-01-01T00:00:00.000Z', {
          zone: 'utc',
        }),
        endDate: DateTime.fromISO('2020-11-30T23:59:59.999Z', {
          zone: 'utc',
        }),
      }
      const expectedResult: Dataload[] = [
        {
          date: DateTime.fromISO('2018-01-01T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 125.25,
          state: DataloadState.VALID,
          valueDetail: null,
        },
        {
          date: DateTime.fromISO('2019-01-01T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 220.5,
          state: DataloadState.VALID,
          valueDetail: null,
        },
        {
          date: DateTime.fromISO('2020-01-01T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 130.33,
          state: DataloadState.VALID,
          valueDetail: null,
        },
      ]
      const mockQueryResult: QueryResult<DataloadEntity[]> = {
        data: loadYearData,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValue(mockQueryResult)
      const result: Dataload[] | null = await queryRunner.fetchFluidData(
        mockTimePeriod,
        TimeStep.YEAR,
        FluidType.WATER
      )
      expect(result).toEqual(expectedResult)
    })

    it('should return the data of the water fluid and month time step', async () => {
      const mockTimePeriod = {
        startDate: DateTime.fromISO('2020-01-01T00:00:00.000Z', {
          zone: 'utc',
        }),
        endDate: DateTime.fromISO('2020-11-30T23:59:59.999Z', {
          zone: 'utc',
        }),
      }
      const expectedResult: Dataload[] = [
        {
          date: DateTime.fromISO('2020-09-01T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 125.25,
          state: DataloadState.VALID,
          valueDetail: null,
        },
        {
          date: DateTime.fromISO('2020-10-01T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 220.5,
          state: DataloadState.VALID,
          valueDetail: null,
        },
        {
          date: DateTime.fromISO('2020-11-01T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 130.33,
          state: DataloadState.VALID,
          valueDetail: null,
        },
      ]
      const mockQueryResult: QueryResult<DataloadEntity[]> = {
        data: loadMonthData,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValue(mockQueryResult)
      const result: Dataload[] | null = await queryRunner.fetchFluidData(
        mockTimePeriod,
        TimeStep.MONTH,
        FluidType.WATER
      )
      expect(result).toEqual(expectedResult)
    })

    it('should return the data of the water fluid and day time step', async () => {
      const mockTimePeriod = {
        startDate: DateTime.fromISO('2020-11-01T00:00:00.000Z', {
          zone: 'utc',
        }),
        endDate: DateTime.fromISO('2020-11-30T23:59:59.999Z', {
          zone: 'utc',
        }),
      }
      const expectedResult: Dataload[] = [
        {
          date: DateTime.fromISO('2020-11-01T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 25.25,
          price: 0.12,
          state: DataloadState.VALID,
          valueDetail: null,
        },
        {
          date: DateTime.fromISO('2020-11-02T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 20.5,
          price: 0.14,
          state: DataloadState.VALID,
          valueDetail: null,
        },
        {
          date: DateTime.fromISO('2020-11-03T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 30.33,
          price: 0.18,
          state: DataloadState.VALID,
          valueDetail: null,
        },
        {
          date: DateTime.fromISO('2020-11-04T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 1.22,
          price: 0.16,
          state: DataloadState.VALID,
          valueDetail: null,
        },
      ]
      const mockQueryResult: QueryResult<DataloadEntity[]> = {
        data: loadDayData,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValue(mockQueryResult)
      const result: Dataload[] | null = await queryRunner.fetchFluidData(
        mockTimePeriod,
        TimeStep.DAY,
        FluidType.WATER
      )
      expect(result).toEqual(expectedResult)
    })

    it('should return the data of the water fluid and week time step', async () => {
      const mockTimePeriod = {
        startDate: DateTime.fromISO('2020-11-02T00:00:00.000Z', {
          zone: 'utc',
        }),
        endDate: DateTime.fromISO('2020-11-08T23:59:59.999Z', {
          zone: 'utc',
        }),
      }
      const expectedResult: Dataload[] = [
        {
          date: DateTime.fromISO('2020-11-02T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 20.5,
          price: 0.14,
          state: DataloadState.VALID,
          valueDetail: null,
        },
        {
          date: DateTime.fromISO('2020-11-03T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 30.33,
          price: 0.18,
          state: DataloadState.VALID,
          valueDetail: null,
        },
        {
          date: DateTime.fromISO('2020-11-04T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 1.22,
          price: 0.16,
          state: DataloadState.VALID,
          valueDetail: null,
        },
      ]
      const mockQueryResult: QueryResult<DataloadEntity[]> = {
        data: loadDayData,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValue(mockQueryResult)
      const result: Dataload[] | null = await queryRunner.fetchFluidData(
        mockTimePeriod,
        TimeStep.WEEK,
        FluidType.WATER
      )
      expect(result).toEqual(expectedResult)
    })

    it('should return the day data of the water fluid and unkown time step', async () => {
      const mockTimePeriod = {
        startDate: DateTime.fromISO('2020-11-01T00:00:00.000Z', {
          zone: 'utc',
        }),
        endDate: DateTime.fromISO('2020-11-30T23:59:59.999Z', {
          zone: 'utc',
        }),
      }
      const expectedResult: Dataload[] = [
        {
          date: DateTime.fromISO('2020-11-01T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 25.25,
          price: 0.12,
          state: DataloadState.VALID,
          valueDetail: null,
        },
        {
          date: DateTime.fromISO('2020-11-02T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 20.5,
          price: 0.14,
          state: DataloadState.VALID,
          valueDetail: null,
        },
        {
          date: DateTime.fromISO('2020-11-03T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 30.33,
          price: 0.18,
          state: DataloadState.VALID,
          valueDetail: null,
        },
        {
          date: DateTime.fromISO('2020-11-04T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 1.22,
          price: 0.16,
          state: DataloadState.VALID,
          valueDetail: null,
        },
      ]
      const mockQueryResult: QueryResult<DataloadEntity[]> = {
        data: loadDayData,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValue(mockQueryResult)
      const result: Dataload[] | null = await queryRunner.fetchFluidData(
        mockTimePeriod,
        99,
        FluidType.WATER
      )
      expect(result).toEqual(expectedResult)
    })

    it('should return the data of the gas fluid and year time step', async () => {
      const mockTimePeriod = {
        startDate: DateTime.fromISO('2018-01-01T00:00:00.000Z', {
          zone: 'utc',
        }),
        endDate: DateTime.fromISO('2020-11-30T23:59:59.999Z', {
          zone: 'utc',
        }),
      }
      const expectedResult: Dataload[] = [
        {
          date: DateTime.fromISO('2018-01-01T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 125.25,
          state: DataloadState.VALID,
          valueDetail: null,
        },
        {
          date: DateTime.fromISO('2019-01-01T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 220.5,
          state: DataloadState.VALID,
          valueDetail: null,
        },
        {
          date: DateTime.fromISO('2020-01-01T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 130.33,
          state: DataloadState.VALID,
          valueDetail: null,
        },
      ]
      const mockQueryResult: QueryResult<DataloadEntity[]> = {
        data: loadYearData,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValue(mockQueryResult)
      const result: Dataload[] | null = await queryRunner.fetchFluidData(
        mockTimePeriod,
        TimeStep.YEAR,
        FluidType.GAS
      )
      expect(result).toEqual(expectedResult)
    })

    it('should return the data of the gas fluid and month time step', async () => {
      const mockTimePeriod = {
        startDate: DateTime.fromISO('2020-01-01T00:00:00.000Z', {
          zone: 'utc',
        }),
        endDate: DateTime.fromISO('2020-11-30T23:59:59.999Z', {
          zone: 'utc',
        }),
      }
      const expectedResult: Dataload[] = [
        {
          date: DateTime.fromISO('2020-09-01T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 125.25,
          state: DataloadState.VALID,
          valueDetail: null,
        },
        {
          date: DateTime.fromISO('2020-10-01T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 220.5,
          state: DataloadState.VALID,
          valueDetail: null,
        },
        {
          date: DateTime.fromISO('2020-11-01T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 130.33,
          state: DataloadState.VALID,
          valueDetail: null,
        },
      ]
      const mockQueryResult: QueryResult<DataloadEntity[]> = {
        data: loadMonthData,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValue(mockQueryResult)
      const result: Dataload[] | null = await queryRunner.fetchFluidData(
        mockTimePeriod,
        TimeStep.MONTH,
        FluidType.GAS
      )
      expect(result).toEqual(expectedResult)
    })

    it('should return the data of the gas fluid and day time step', async () => {
      const mockTimePeriod = {
        startDate: DateTime.fromISO('2020-11-01T00:00:00.000Z', {
          zone: 'utc',
        }),
        endDate: DateTime.fromISO('2020-11-30T23:59:59.999Z', {
          zone: 'utc',
        }),
      }
      const expectedResult: Dataload[] = [
        {
          date: DateTime.fromISO('2020-11-01T00:00:00.000Z', {
            zone: 'utc',
          }),
          price: 0.12,
          value: 25.25,
          state: DataloadState.VALID,
          valueDetail: null,
        },
        {
          date: DateTime.fromISO('2020-11-02T00:00:00.000Z', {
            zone: 'utc',
          }),
          price: 0.14,
          value: 20.5,
          state: DataloadState.VALID,
          valueDetail: null,
        },
        {
          date: DateTime.fromISO('2020-11-03T00:00:00.000Z', {
            zone: 'utc',
          }),
          price: 0.18,
          value: 30.33,
          state: DataloadState.VALID,
          valueDetail: null,
        },
        {
          date: DateTime.fromISO('2020-11-04T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 1.22,
          price: 0.16,
          state: DataloadState.VALID,
          valueDetail: null,
        },
      ]
      const mockQueryResult: QueryResult<DataloadEntity[]> = {
        data: loadDayData,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValue(mockQueryResult)
      const result: Dataload[] | null = await queryRunner.fetchFluidData(
        mockTimePeriod,
        TimeStep.DAY,
        FluidType.GAS
      )
      expect(result).toEqual(expectedResult)
    })

    it('should return the data of the gas fluid and week time step', async () => {
      const mockTimePeriod = {
        startDate: DateTime.fromISO('2020-11-02T00:00:00.000Z', {
          zone: 'utc',
        }),
        endDate: DateTime.fromISO('2020-11-08T23:59:59.999Z', {
          zone: 'utc',
        }),
      }
      const expectedResult: Dataload[] = [
        {
          date: DateTime.fromISO('2020-11-02T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 20.5,
          price: 0.14,
          state: DataloadState.VALID,
          valueDetail: null,
        },
        {
          date: DateTime.fromISO('2020-11-03T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 30.33,
          price: 0.18,
          state: DataloadState.VALID,
          valueDetail: null,
        },
        {
          date: DateTime.fromISO('2020-11-04T00:00:00.000Z', {
            zone: 'utc',
          }),
          price: 0.16,
          value: 1.22,
          state: DataloadState.VALID,
          valueDetail: null,
        },
      ]
      const mockQueryResult: QueryResult<DataloadEntity[]> = {
        data: loadDayData,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValue(mockQueryResult)
      const result: Dataload[] | null = await queryRunner.fetchFluidData(
        mockTimePeriod,
        TimeStep.WEEK,
        FluidType.GAS
      )
      expect(result).toEqual(expectedResult)
    })

    it('should return the day data of the gas fluid and unkonwn time step', async () => {
      const mockTimePeriod = {
        startDate: DateTime.fromISO('2020-11-01T00:00:00.000Z', {
          zone: 'utc',
        }),
        endDate: DateTime.fromISO('2020-11-30T23:59:59.999Z', {
          zone: 'utc',
        }),
      }
      const expectedResult: Dataload[] = [
        {
          date: DateTime.fromISO('2020-11-01T00:00:00.000Z', {
            zone: 'utc',
          }),
          price: 0.12,
          value: 25.25,
          state: DataloadState.VALID,
          valueDetail: null,
        },
        {
          date: DateTime.fromISO('2020-11-02T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 20.5,
          price: 0.14,
          state: DataloadState.VALID,
          valueDetail: null,
        },
        {
          date: DateTime.fromISO('2020-11-03T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 30.33,
          price: 0.18,
          state: DataloadState.VALID,
          valueDetail: null,
        },
        {
          date: DateTime.fromISO('2020-11-04T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 1.22,
          price: 0.16,
          state: DataloadState.VALID,
          valueDetail: null,
        },
      ]
      const mockQueryResult: QueryResult<DataloadEntity[]> = {
        data: loadDayData,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValue(mockQueryResult)
      const result: Dataload[] | null = await queryRunner.fetchFluidData(
        mockTimePeriod,
        99,
        FluidType.GAS
      )
      expect(result).toEqual(expectedResult)
    })

    it('should return the data of the elec fluid and month time step with start year <> end year', async () => {
      const mockTimePeriod = {
        startDate: DateTime.fromISO('2019-12-01T00:00:00.000Z', {
          zone: 'utc',
        }),
        endDate: DateTime.fromISO('2020-10-30T23:59:59.999Z', {
          zone: 'utc',
        }),
      }
      const expectedResult: Dataload[] = [
        {
          date: DateTime.fromISO('2020-09-01T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 125.25,
          state: DataloadState.VALID,
          valueDetail: null,
        },
        {
          date: DateTime.fromISO('2020-10-01T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 220.5,
          state: DataloadState.VALID,
          valueDetail: null,
        },
      ]
      const mockQueryResult: QueryResult<DataloadEntity[]> = {
        data: [loadMonthData[0], loadMonthData[1]],
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValue(mockQueryResult)
      const result: Dataload[] | null = await queryRunner.fetchFluidData(
        mockTimePeriod,
        TimeStep.MONTH,
        FluidType.ELECTRICITY
      )
      expect(result).toEqual(expectedResult)
    })

    it('should return the data of the elec fluid and month time step with period >=12 months', async () => {
      const mockTimePeriod = {
        startDate: DateTime.fromISO('2019-12-01T00:00:00.000Z', {
          zone: 'utc',
        }),
        endDate: DateTime.fromISO('2020-11-30T23:59:59.999Z', {
          zone: 'utc',
        }),
      }
      const expectedResult: Dataload[] = [
        {
          date: DateTime.fromISO('2020-09-01T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 125.25,
          state: DataloadState.VALID,
          valueDetail: null,
        },
        {
          date: DateTime.fromISO('2020-10-01T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 220.5,
          state: DataloadState.VALID,
          valueDetail: null,
        },
        {
          date: DateTime.fromISO('2020-11-01T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 130.33,
          state: DataloadState.VALID,
          valueDetail: null,
        },
      ]
      const mockQueryResult: QueryResult<DataloadEntity[]> = {
        data: loadMonthData,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValue(mockQueryResult)
      const result: Dataload[] | null = await queryRunner.fetchFluidData(
        mockTimePeriod,
        TimeStep.MONTH,
        FluidType.ELECTRICITY
      )
      expect(result).toEqual(expectedResult)
    })

    it('should return the data of the elec fluid and day time step with start month <> end month', async () => {
      const mockTimePeriod = {
        startDate: DateTime.fromISO('2020-10-15T00:00:00.000Z', {
          zone: 'utc',
        }),
        endDate: DateTime.fromISO('2020-11-05T23:59:59.999Z', {
          zone: 'utc',
        }),
      }
      const expectedResult: Dataload[] = [
        {
          date: DateTime.fromISO('2020-11-01T00:00:00.000Z', {
            zone: 'utc',
          }),
          price: 0.12,
          value: 25.25,
          state: DataloadState.VALID,
          valueDetail: null,
        },
        {
          date: DateTime.fromISO('2020-11-02T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 20.5,
          price: 0.14,
          state: DataloadState.VALID,
          valueDetail: null,
        },
        {
          date: DateTime.fromISO('2020-11-03T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 30.33,
          price: 0.18,
          state: DataloadState.VALID,
          valueDetail: null,
        },
        {
          date: DateTime.fromISO('2020-11-04T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 1.22,
          price: 0.16,
          state: DataloadState.VALID,
          valueDetail: null,
        },
      ]
      const mockQueryResult: QueryResult<DataloadEntity[]> = {
        data: loadDayData,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValue(mockQueryResult)
      const result: Dataload[] | null = await queryRunner.fetchFluidData(
        mockTimePeriod,
        TimeStep.DAY,
        FluidType.ELECTRICITY
      )
      expect(result).toEqual(expectedResult)
    })

    it('should return the data of the elec fluid and day time step with time period > 31 days', async () => {
      const mockTimePeriod = {
        startDate: DateTime.fromISO('2020-10-01T00:00:00.000Z', {
          zone: 'utc',
        }),
        endDate: DateTime.fromISO('2020-11-30T23:59:59.999Z', {
          zone: 'utc',
        }),
      }
      const expectedResult: Dataload[] = [
        {
          date: DateTime.fromISO('2020-11-01T00:00:00.000Z', {
            zone: 'utc',
          }),
          price: 0.12,
          value: 25.25,
          state: DataloadState.VALID,
          valueDetail: null,
        },
        {
          date: DateTime.fromISO('2020-11-02T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 20.5,
          price: 0.14,
          state: DataloadState.VALID,
          valueDetail: null,
        },
        {
          date: DateTime.fromISO('2020-11-03T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 30.33,
          price: 0.18,
          state: DataloadState.VALID,
          valueDetail: null,
        },
        {
          date: DateTime.fromISO('2020-11-04T00:00:00.000Z', {
            zone: 'utc',
          }),
          value: 1.22,
          price: 0.16,
          state: DataloadState.VALID,
          valueDetail: null,
        },
      ]
      const mockQueryResult: QueryResult<DataloadEntity[]> = {
        data: loadDayData,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValue(mockQueryResult)
      const result: Dataload[] | null = await queryRunner.fetchFluidData(
        mockTimePeriod,
        TimeStep.DAY,
        FluidType.ELECTRICITY
      )
      expect(result).toEqual(expectedResult)
    })

    it('should return null when fetch data failed', async () => {
      const mockTimePeriod = {
        startDate: DateTime.fromISO('2020-11-01T00:00:00.000Z', {
          zone: 'utc',
        }),
        endDate: DateTime.fromISO('2020-11-30T23:59:59.999Z', {
          zone: 'utc',
        }),
      }
      mockClient.query.mockRejectedValue(new Error())
      const result: Dataload[] | null = await queryRunner.fetchFluidData(
        mockTimePeriod,
        TimeStep.DAY,
        FluidType.ELECTRICITY
      )
      expect(result).toBeNull()
    })

    it('should return null when unknown fluid type', async () => {
      const mockTimePeriod = {
        startDate: DateTime.fromISO('2020-11-01T00:00:00.000Z', {
          zone: 'utc',
        }),
        endDate: DateTime.fromISO('2020-11-30T23:59:59.999Z', {
          zone: 'utc',
        }),
      }
      const result: Dataload[] | null = await queryRunner.fetchFluidData(
        mockTimePeriod,
        TimeStep.DAY,
        99
      )
      expect(result).toBeNull()
    })

    it('should return null when unknown time step', async () => {
      const mockTimePeriod = {
        startDate: DateTime.fromISO('2020-11-01T00:00:00.000Z', {
          zone: 'utc',
        }),
        endDate: DateTime.fromISO('2020-11-30T23:59:59.999Z', {
          zone: 'utc',
        }),
      }
      const result: Dataload[] | null = await queryRunner.fetchFluidData(
        mockTimePeriod,
        99,
        FluidType.ELECTRICITY
      )
      expect(result).toBeNull()
    })
  })

  describe('fetchFluidMaxData method', () => {
    it('should return the max load for elec fluid with day timestep', async () => {
      const mockTimePeriod = {
        startDate: DateTime.fromISO('2020-11-01T00:00:00.000Z', {
          zone: 'utc',
        }),
        endDate: DateTime.fromISO('2020-11-30T23:59:59.999Z', {
          zone: 'utc',
        }),
      }
      const mockQueryResult: QueryResult<DataloadEntity[]> = {
        data: [...loadDayData].sort((a, b) => b.load - a.load),
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValue(mockQueryResult)
      const result: number | Dataload | null =
        await queryRunner.fetchFluidMaxData(
          mockTimePeriod,
          TimeStep.DAY,
          FluidType.ELECTRICITY
        )
      expect(result).toBe(30.33)
    })

    it('should return the max load for elec fluid with half an hour timestep', async () => {
      const mockTimePeriod = {
        startDate: DateTime.fromISO('2020-11-02T00:00:00.000Z', {
          zone: 'utc',
        }),
        endDate: DateTime.fromISO('2020-11-2T23:59:59.999Z', {
          zone: 'utc',
        }),
      }
      const mockQueryResult: QueryResult<DataloadEntity[]> = {
        data: [loadMinuteData[4]],
        bookmark: '',
        next: false,
        skip: 0,
      }
      const mockQueryResult2: QueryResult<DataloadEntity[]> = {
        data: [loadMinuteData[0]],
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query
        .mockResolvedValueOnce(mockQueryResult)
        .mockResolvedValueOnce(mockQueryResult2)
      const result: number | Dataload | null =
        await queryRunner.fetchFluidMaxData(
          mockTimePeriod,
          TimeStep.HALF_AN_HOUR,
          FluidType.ELECTRICITY
        )
      expect(result).toBe(7.82)
    })

    it('should return 0 for elec fluid with half an hour timestep and no data found', async () => {
      const mockTimePeriod = {
        startDate: DateTime.fromISO('2020-11-02T00:00:00.000Z', {
          zone: 'utc',
        }),
        endDate: DateTime.fromISO('2020-11-2T23:59:59.999Z', {
          zone: 'utc',
        }),
      }
      const mockQueryResult: QueryResult<DataloadEntity[]> = {
        data: [],
        bookmark: '',
        next: false,
        skip: 0,
      }
      const mockQueryResult2: QueryResult<DataloadEntity[]> = {
        data: [],
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query
        .mockResolvedValueOnce(mockQueryResult)
        .mockResolvedValueOnce(mockQueryResult2)
      const result: number | Dataload | null =
        await queryRunner.fetchFluidMaxData(
          mockTimePeriod,
          TimeStep.HALF_AN_HOUR,
          FluidType.ELECTRICITY
        )
      expect(result).toBe(0)
    })

    it('should return null when fetch data failed', async () => {
      const mockTimePeriod = {
        startDate: DateTime.fromISO('2020-11-01T00:00:00.000Z', {
          zone: 'utc',
        }),
        endDate: DateTime.fromISO('2020-11-30T23:59:59.999Z', {
          zone: 'utc',
        }),
      }
      mockClient.query.mockRejectedValue(new Error())
      const result: number | Dataload | null =
        await queryRunner.fetchFluidMaxData(
          mockTimePeriod,
          TimeStep.DAY,
          FluidType.ELECTRICITY
        )
      expect(result).toBeNull()
    })

    it('should return null when unknown fluid type', async () => {
      const mockTimePeriod = {
        startDate: DateTime.fromISO('2020-11-01T00:00:00.000Z', {
          zone: 'utc',
        }),
        endDate: DateTime.fromISO('2020-11-30T23:59:59.999Z', {
          zone: 'utc',
        }),
      }
      const result: number | Dataload | null =
        await queryRunner.fetchFluidMaxData(mockTimePeriod, TimeStep.DAY, 99)
      expect(result).toBeNull()
    })

    it('should return null when unknown time step', async () => {
      const mockTimePeriod = {
        startDate: DateTime.fromISO('2020-11-01T00:00:00.000Z', {
          zone: 'utc',
        }),
        endDate: DateTime.fromISO('2020-11-30T23:59:59.999Z', {
          zone: 'utc',
        }),
      }
      const result: number | Dataload | null =
        await queryRunner.fetchFluidMaxData(
          mockTimePeriod,
          99,
          FluidType.ELECTRICITY
        )
      expect(result).toBeNull()
    })
  })

  describe('getFirstDateData method', () => {
    it('should return the first load for elec fluid', async () => {
      const firstMockData = [...loadDayData]
      firstMockData.sort(
        (a, b) =>
          (a.year - b.year) * 100 + (a.month - b.month) * 10 + (a.day - b.day)
      )
      const mockQueryResult: QueryResult<DataloadEntity[]> = {
        data: [firstMockData[0]],
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValue(mockQueryResult)
      const result = await queryRunner.getFirstDateData(FluidType.ELECTRICITY)
      expect(result).toEqual(
        DateTime.local(
          loadDayData[0].year,
          loadDayData[0].month,
          loadDayData[0].day
        ).setZone('utc', { keepLocalTime: true })
      )
    })

    it('should return null when fetch data failed', async () => {
      mockClient.query.mockRejectedValue(new Error())
      const result = await queryRunner.getFirstDateData(FluidType.ELECTRICITY)
      expect(result).toBeNull()
    })
  })

  describe('getLastDateData method', () => {
    it('should return the last load for elec fluid', async () => {
      const lastMockData = [...loadDayData]
      lastMockData.sort(
        (a, b) =>
          (b.year - a.year) * 100 + (b.month - a.month) * 10 + (b.day - a.day)
      )
      const mockQueryResult: QueryResult<DataloadEntity[]> = {
        data: [lastMockData[0]],
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValue(mockQueryResult)
      const result = await queryRunner.getLastDateData(FluidType.ELECTRICITY)
      expect(result).toEqual(
        DateTime.local(
          loadDayData[3].year,
          loadDayData[3].month,
          loadDayData[3].day
        ).setZone('utc', { keepLocalTime: true })
      )
    })

    it('should return null when fetch data failed', async () => {
      mockClient.query.mockRejectedValue(new Error())
      const result = await queryRunner.getLastDateData(FluidType.ELECTRICITY)
      expect(result).toBeNull()
    })
  })

  describe('getEntries method', () => {
    it('should return query result for elec fluid with day timestep', async () => {
      const mockQueryResult: QueryResult<DataloadEntity[]> = {
        data: loadDayData,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValue(mockQueryResult)
      const result = await queryRunner.getEntries(
        FluidType.ELECTRICITY,
        TimeStep.DAY
      )
      expect(result).toEqual(mockQueryResult)
    })

    it('should return null when fetch data failed', async () => {
      mockClient.query.mockRejectedValue(new Error())
      const result = await queryRunner.getEntries(
        FluidType.ELECTRICITY,
        TimeStep.DAY
      )
      expect(result).toBeNull()
    })

    it('should return null when unknown fluid type', async () => {
      const result = await queryRunner.getEntries(99, TimeStep.DAY)
      expect(result).toBeNull()
    })

    it('should return null when unknown time step', async () => {
      const result = await queryRunner.getEntries(FluidType.ELECTRICITY, 99)
      expect(result).toBeNull()
    })
  })

  describe('fetchFluidData method', () => {
    it('should return the data of the elec fluid and year time step', async () => {
      const mockTimePeriod = {
        startDate: DateTime.fromISO('2018-01-01T00:00:00.000Z', {
          zone: 'utc',
        }),
        endDate: DateTime.fromISO('2020-11-30T23:59:59.999Z', {
          zone: 'utc',
        }),
      }
      const mockQueryResult: QueryResult<DataloadEntity[]> = {
        data: loadYearData,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValue(mockQueryResult)
      const result: QueryResult<any> = await queryRunner.fetchFluidRawDoctype(
        mockTimePeriod,
        TimeStep.YEAR,
        FluidType.ELECTRICITY
      )
      expect(result).toEqual(mockQueryResult)
      expect(result.data.length).toEqual(3)
    })
  })
})
