import { DateTime } from 'luxon'
import mockClient from '../../tests/__mocks__/client'
import UsageEventService from './usageEvent.service'
import { AddEventParams, UsageEventEntity } from 'models'
import { QueryResult } from 'cozy-client'
import {
  allUsageEventsData,
  connectionAttemptEGLError,
  connectionAttemptEGLSuccess,
  connectionEventEntitiesData,
  connectionUsageEventsData,
  usageEventData,
  usageEventEntityData,
} from '../../tests/__mocks__/usageEventsData.mock'
import { UsageEventType } from 'enum/usageEvent.enum'

const localSpy = jest.spyOn(DateTime, 'local')

describe('UsageEvent service', () => {
  beforeEach(() => {
    localSpy.mockClear()
  })
  describe('addEvent method', () => {
    it('should add event and return it', async () => {
      const { eventDate, ...addEventParams } = usageEventEntityData
      const params: AddEventParams = {
        ...addEventParams,
        startDate: addEventParams.startDate
          ? DateTime.fromISO(addEventParams.startDate, {
              zone: 'utc',
            })
          : undefined,
      }
      const mockDate = DateTime.fromISO(eventDate, {
        zone: 'utc',
      })
      const mockQueryResult: QueryResult<UsageEventEntity> = {
        data: usageEventEntityData,
        bookmark: '',
        next: false,
        skip: 0,
      }
      localSpy.mockReturnValue(mockDate)
      mockClient.create.mockResolvedValueOnce(mockQueryResult)
      const result = await UsageEventService.addEvent(mockClient, params)
      expect(result).toEqual(usageEventData)
    })
  })
  describe('updateUsageEventsAggregated method', () => {
    it('should return true Connection events', async () => {
      const mockQueryResult: QueryResult<UsageEventEntity> = {
        data: usageEventEntityData,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.save.mockResolvedValue(mockQueryResult)
      const result = await UsageEventService.updateUsageEventsAggregated(
        mockClient,
        allUsageEventsData
      )
      expect(result).toEqual(true)
    })
    it('should throw an error', async () => {
      mockClient.save.mockRejectedValue(new Error())
      try {
        await UsageEventService.updateUsageEventsAggregated(
          mockClient,
          allUsageEventsData
        )
      } catch (error) {
        expect(error).toEqual(new Error())
      }
    })
  })
  describe('getEvents method', () => {
    it('should return all Connection events', async () => {
      const mockQueryResult: QueryResult<UsageEventEntity[]> = {
        data: connectionEventEntitiesData,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValueOnce(mockQueryResult)
      const result = await UsageEventService.getEvents(mockClient, {
        type: UsageEventType.CONNECTION_EVENT,
      })
      expect(result).toEqual(connectionUsageEventsData)
    })
  })
  describe('addEventIfDoesntExist method', () => {
    it('should not add event', async () => {
      const mockQueryResult: QueryResult<UsageEventEntity[]> = {
        data: connectionEventEntitiesData,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValueOnce(mockQueryResult)
      const result = await UsageEventService.addEventIfDoesntExist(
        mockClient,
        { type: UsageEventType.CONNECTION_EVENT },
        { id: { $gt: 0 } }
      )
      expect(result).toEqual(null)
    })
    it('should add event', async () => {
      const mockQueryResult: QueryResult<UsageEventEntity[]> = {
        data: [],
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValueOnce(mockQueryResult)
      const mockQueryResult2: QueryResult<UsageEventEntity> = {
        data: usageEventEntityData,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.create.mockResolvedValueOnce(mockQueryResult2)
      const result = await UsageEventService.addEventIfDoesntExist(
        mockClient,
        { type: UsageEventType.CONNECTION_EVENT },
        { id: { $gt: 0 } }
      )
      expect(result).toEqual(usageEventData)
    })
  })
  describe('udpateConnectionAttemptEvent method', () => {
    it('should update the last attempt to true', async () => {
      const mockQueryResult: QueryResult<UsageEventEntity[]> = {
        data: [connectionAttemptEGLError],
        bookmark: '',
        next: false,
        skip: 0,
      }
      const mockQueryResult2: QueryResult<UsageEventEntity> = {
        data: connectionAttemptEGLSuccess,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValueOnce(mockQueryResult)
      mockClient.save.mockResolvedValueOnce(mockQueryResult2)
      const result = await UsageEventService.udpateConnectionAttemptEvent(
        mockClient,
        'eglgrandlyon'
      )
      expect(result).toEqual(connectionAttemptEGLSuccess)
    })
    it('should fail to update the attempts', async () => {
      const mockQueryResult: QueryResult<UsageEventEntity[] | undefined> = {
        data: undefined,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValueOnce(mockQueryResult)
      const result = await UsageEventService.udpateConnectionAttemptEvent(
        mockClient,
        'eglgrandlyon'
      )
      expect(result).toEqual(undefined)
    })
    it('should find no attempt', async () => {
      const mockQueryResult: QueryResult<UsageEventEntity[] | undefined> = {
        data: [],
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValueOnce(mockQueryResult)
      const result = await UsageEventService.udpateConnectionAttemptEvent(
        mockClient,
        'eglgrandlyon'
      )
      expect(result).toEqual(undefined)
    })
  })
})
